@@ -392,7 +392,7 @@ class XTypeError(Error, TypeError):
     """A TypeError containing the expected and actual types."""
 
 
-@ErrorUtils.args(value=KeyLike)
+@ErrorUtils.args(value=Any)
 class XValueError(Error, ValueError):
     """A ValueError containing the value."""
 
@@ -531,6 +531,10 @@ class IllegalPathError(PathError, ValueError):
     """Not a valid path (e.g. not ok on the filesystem)."""
 
 
+class InvalidFileType(PathError):
+    """Not a valid file type."""
+
+
 class FileDoesNotExistError(PathError):
     """A file is expected, but the path does not exist."""
 
